
// Gives us access to all functions in the 'system' namespace
using System;


class NormalClass {

	// a property will map to this value
	private int a_value;
	
	// constructor
	public NormalClass(){
		Console.Write("\nBase constructor called");
	}
	
	// declare a property for 'a_value'
	protected int A_Value
	{
		get
		{
			return a_value;
		}
		set
		{
			a_value = value;
		}
	}
	
	// A virtual function which can be overriden
	// by a derived class
	// Note the protected access which gives only
	// objects of this class and objects derived from
	// this class access to the methods
	protected virtual void VirtualFunction(){
		Console.Write("\nThis is base VirtualFunction 1");
	}
	
	protected virtual void VirtualFunction2(){
		Console.Write("\nThis is base VirtualFunction 2");
	}
	
	// A normal method
	protected void NormalMethod(){
		Console.Write("\nThis is base NormalMethod");
	}
	
	public void CallProtectedMethods(string password, int newPropertyValue){
	
		if (password=="12345") 
		{
			VirtualFunction();
			VirtualFunction2();
			NormalMethod();
			A_Value = newPropertyValue;
		}
		else
		{
			Console.Write("\nINVALID");
		}
	}
	
	public void showValue(string password){
	
		if (password=="12345") 
		{
			Console.Write("\nValue is now: {0}", A_Value);
		}
		else
		{
			Console.Write("\nINVALID");
		}
	}
}

class InheritsNormal : NormalClass {

	// Constructor
	public InheritsNormal(){
		Console.Write("\nClient constructor called");
	}

	// Add a new method in addition to the two inherited
	// This method demonstrates how we can force a call
	// to a virtual function in an object's base class
	public void demonstrateBaseCall(){
		base.VirtualFunction();
	}
	
	// Now we override the two virtual methods
	// ------------------------------------------------------>
	
	protected override void VirtualFunction(){
		Console.Write("\nThis is derived VirtualFunction 1");
	}
	
	//public override void VirtualFunction2(){
		//Console.Write("\nThis is derived VirtualFunction 2");
	//}
	
	// <------------------------------------------------------

	// Finally we try and implement a new version of 'NormalMethod'
	// by a process known as HIDING which can only happen when
	// the base class doesn't declare its version of the function
	// as virtual:
	
	protected new void NormalMethod(){
		Console.Write("\nThis is derived NormalMethod");
	}
}


class GettingStuckIntoClasses3{
		
	public static void Main(){	
	
		// declare an instance of 'InheritFromMe' class, and
		// an instance of 'Inherits'.		
		NormalClass baseInstance = new NormalClass();
		InheritsNormal derivedClass = new InheritsNormal();

		// Even though we have instances of the two classes
		// we still can't access their protected methods or properties.
		// The only entities that can access the protected
		// methods of the NormalClass are the actual
		// instances of the class NormalClass and all
		// derived classes.
		// They have a method which can do that, which
		// implements a password.  If we know the password
		// it will execute the protected methods and change
		// the property on our behalf as shown:
		
		// display property
		baseInstance.showValue("12345");
		
		// try and change new property with correct password
		int newPropertyValue = 10;
		baseInstance.CallProtectedMethods("12345", newPropertyValue);
		
		// show result
		Console.Write("\nValue should be changed now");
		baseInstance.showValue("12345");
		
		// if we don't know the password, this happens:
		Console.Write("\nGuessing password...");
		baseInstance.CallProtectedMethods("a guess", newPropertyValue);
		
		// The above example maps well to any data situation, where
		// a password or pin number would be required to allow
		// you to view or change data
				
	}
}
